-- this file demonstrates the problem with foreign keys
-- described in the book on pages 54-55.

-- This soultion solves part of the problem by using a transaction
-- for the insertions, as hinted at on page 55.

-- This doesn't work in MySQL. According to 
--    http://dev.mysql.com/doc/refman/5.0/en/innodb-foreign-key-constraints.html
-- "in MySQL, foreign key constraints are checked immediately" so there is no
-- deferred constrained checking for foreign keys.


SET CONSTRAINTS ALL DEFERRED;
SET AUTOCOMMIT = 0;


CREATE TABLE A ( id INTEGER,                  
                 ref INTEGER, 
                 PRIMARY KEY (id) 
--                 ,FOREIGN KEY (ref) REFERENCES B(id)
               ) ENGINE=InnoDB;


CREATE TABLE B ( id INTEGER,                  
                 ref INTEGER, 
                 PRIMARY KEY (id), 
                 FOREIGN KEY (ref) REFERENCES A(id)
               ) ENGINE=InnoDB;

-- put the foreign key constraint back in table A
ALTER TABLE A ADD 
   CONSTRAINT fk
      FOREIGN KEY (ref) REFERENCES B(id);
      
      
-- these insert statements shouldn't fail when they are in a single transaction
START TRANSACTION;
INSERT INTO A VALUES( 1, 2 );
INSERT INTO A VALUES( 2, 2 );
INSERT INTO B VALUES( 1, 1 );
INSERT INTO B VALUES( 2, 1 );
COMMIT;

select * from A;
select * from B;


-- remove the foreign key constraint from table A
-- so that we can drop the tables
ALTER TABLE A DROP FOREIGN KEY fk;

DROP TABLE IF EXISTS B;
DROP TABLE IF EXISTS A;
